@@ -6,7 +6,8 @@ __email__ = "dariusz.dwornikowski@cs.put.poznan.pl"
 
 from pprint import pprint
 from pypepa import PEPAModel
-from pypepa.utils import pretty_print_vector, pretty_print_performance
+from pypepa.utils import pretty_print_vector, pretty_print_performance,\
+                           pretty_print_utilisations
 from pypepa.experiments.experiment import rate_experiment, range_maker,\
                                             rate_experiment_two
 from pypepa.experiments.graphing import plot_2d, plot_3d
@@ -14,6 +15,7 @@ from pypepa.logger import init_log
 from pypepa import __version__
 import argparse
 import sys
+import os
 
 
 def main():
@@ -35,25 +37,29 @@ def main():
                           choices=['direct', 'sparse'],
                           help="choose solver type DEFAULT: sparse",
                           default="sparse")
+    sol_args.add_argument("-da", "--derive-algorithm", action="store",
+                          type=str, choices=["BU", "TD"],
+                          help="Choose the algorithm to use for deriving the statespace",
+                          default="BU")
     output_args = parser.add_argument_group("Output", "Output based options")
     parser.add_argument("file", help="path to the model file")
-    output_args.add_argument("-gd", "--generate_dots",
+    output_args.add_argument("-gd", "--generate_dots", nargs='?', default="",
                              help="generate a graphviz dot file for every"
                                   "sequential component in a GENDOTS folder and"
-                                  "for the whole state space",
+                                  "for the whole state space", const='dots',
                              action="store", dest="gendots", type=str)
     output_args.add_argument("-st", "--steady",
                              help="print steady state probability vector",
                              action="store_true")
+    output_args.add_argument("-ut", "--utilisations",
+                             help="print steady state utilisations",
+                             action="store_true")
     output_args.add_argument("-th", "--performance",
-                             help="print throughoutput of actions",
+                             help="print throughput of actions",
                              action="store_true", dest="top")
     output_args.add_argument("-tr", "--transient",
-                             help="print throughoutput of actions",
+                             help="print throughput of actions",
                              action="store", dest="trantime", type=int)
-    # output_args.add_argument("-ut", "--utilization",
-                               # help="print utilization of action",
-                               # action="store_true", dest="util")
     output_args.add_argument("-f", "--format", dest="format", type=str,
                              choices=["graph", "console", "csv"],
                              help="format for -st -th -varrate", default="console")
@@ -66,23 +72,18 @@ def main():
                                "or rate:RATENAME:l:val1,val2,val3",
                           action="append", dest="variables")
     exp_args.add_argument("-val", "--value", action="store", dest="yvar") 
-    # exp_args.add_argument("--actionth",
-    #                       help="throughoutput of action on the Y axis",
-    #                       dest="actionth", action="store",
-    #                       metavar="action name")
 
     args = parser.parse_args()
 
     logger = init_log(log_level=args.loglevel)
-    pargs = {"file": args.file, "solver" : args.solver}
-
+    pargs = {"file": args.file, "solver" : args.solver,
+             "derive_algorithm": args.derive_algorithm}
     if args.gendots:
         try:
             pm = PEPAModel(**pargs)
         except Exception as e:
             print("Exception occured:", e)
             sys.exit(1)
-        import os
         if os.path.isdir(args.gendots):
             pass
         else:
@@ -114,38 +115,46 @@ def main():
             sys.exit(1)
     try:
         pm = PEPAModel(**pargs)
+        name = os.path.splitext(args.output)[0] \
+                    if args.output else os.path.splitext(pm.name)
     except Exception as e:
+        raise
         print("Exception occured: ", e)
         sys.exit(1)
 
-
-
-    if args.steady or args.top:
+    if args.steady or args.top or args.utilisations:
         pm.steady_state()
         print("Statespace of {} has {} states \n".format(args.file,
               len(pm.get_steady_state_vector() )))
+
     if args.trantime:
-        tr = pm.transient(0, int(args.trantime))
         print("Transient analysis from time %d to %d" % (0, args.trantime))
-        args.output = "{}-transient.csv".format(pm.name)
-        pretty_print_vector(tr,
+        args.output = "{}-transient.csv".format(name)
+        pretty_print_vector(pm.transient(0, int(args.trantime)),
                              pm.get_state_names(),
                              fmt=args.format,
                              outfile=args.output
                              )
     if args.steady:
         print("Steady state vector")
-        args.output = "{}-steady.csv".format(pm.name)
+        args.output = "{}-steady.csv".format(name)
         pretty_print_vector(pm.get_steady_state_vector(),
                              pm.get_state_names(),
                              fmt=args.format,
                              outfile=args.output
                              )
+    if args.utilisations:
+        print ("Steady State utilisations")
+        args.output = "{}-utilisations.csv".format(name)
+        pretty_print_utilisations(pm.get_utilisations(),
+                                  fmt=args.format,
+                                  outfile=args.output
+                                 )
     if args.top:
-        print("Throuhoutput (successful action completion in one time unit)")
+        print("Throughput (successful action completion in a time unit)")
         print("Output:{}".format(args.format))
-        args.output = "{}-throughput.csv".format(pm.name)
-        pretty_print_performance(pm.get_throughoutput(), fmt=args.format,
+        args.output = "{}-throughput.csv".format(name)
+        pretty_print_performance(pm.get_throughput(), fmt=args.format,
                                                          outfile=args.output)
 
 
