@@ -1,28 +1,30 @@
 #!/usr/bin/env python
-"""
-PEPA Parser
-"""
-from pyparsing import Word, Literal, alphas, alphanums, nums, Combine, Optional, ZeroOrMore, Forward, restOfLine
-import sys
-from pypepa.parsing.pepa_ast import *
+from pyparsing import Word, Literal, alphas, alphanums, Combine, Optional,\
+                      ZeroOrMore, Forward, restOfLine, nums, lineno, col, \
+                      ParseException
+from pypepa.parsing.pepa_ast import ActivityNode, ProcdefNode, DefNode, CoopNode, \
+                                    PrefixNode, SyncsetNode
+from pypepa.parsing.rate_parser import RateParser
 from pypepa.logger import init_log
+from pypepa.exceptions import VariableNotDefinedError, VariableAlreadyDefinedError, \
+                               ProcessNotDefinedError, ProcessNotDefinedError, \
+                               ProcessAlreadyDefinedError, InvalidActionTypeError
+import pprint
 
 class PEPAParser(object):
-    """
-    TODO: change to dicts
-    """
 
     def __init__(self):
         self.log_pa = init_log()
         self._processes = {}
         self._var_stack = {}
         self._actions = []
+        self._seen = {}
         self.systemeq = None
-
+        self.rate_parser = RateParser()
 
     def _create_activity(self, string, loc,tok):
         self.log_pa.debug("Token: "+tok[0])
-        n = ActivityNode("(" + tok[0] + "," + tok[1] + ")", "activity")
+        n = ActivityNode("(" + tok[0] + "," + tok[1] + ")")
         n.action = tok[0]
         self._actions.append(n.action)
         n.rate = tok[1]
@@ -30,7 +32,8 @@ class PEPAParser(object):
 
     def _create_procdef(self, s, loc, toks):
         self.log_pa.debug("Token: "+toks[0])
-        n = ProcdefNode(toks[0], "procdef")
+        n = ProcdefNode(toks[0])
+        self._seen[toks[0]] = (loc, s)
         if len(toks) > 1:
             n.aggregation = True
             if toks[1] in self._var_stack:
@@ -43,49 +46,44 @@ class PEPAParser(object):
     def _create_definition(self, string, loc, tok):
         self.log_pa.debug("Left token: "+tok[0].data)
         self.log_pa.debug("Right token: "+tok[2].data)
-        n = DefNode("=", "definition")
+        n = DefNode("=")
         n.left = tok[0]
         n.right = tok[2]
         n.process = tok[0].data
-        for key in self._processes.keys():
-            if self._processes[key].process == tok[0].data:
-                self.log_pa.error("Process "+tok[0].data+" already defined")
-                exit(1)
-        self._processes[tok[0]] = n
+        if n.process in self._processes:
+            message = "Process "+n.process+" already defined"
+            self.log_pa.error(message)
+            raise ProcessAlreadyDefinedError(message)
+        self._processes[n.process] = n
         return n
 
     def _create_prefix(self,string, loc, tok):
-        self.log_pa.debug("Tokens: "+str( len(tok) ))
+        self.log_pa.debug("Tokens: " + str(len(tok)))
         if len(tok) > 1:
-            self.log_pa.debug("Left token: "+tok[0].data)
-            self.log_pa.debug("Right token: "+tok[2].data)
-            n = PrefixNode(".", "prefix")
-            lhs = tok[0]
-            rhs = tok[2]
-            n.left = lhs
-            n.right = rhs
+            self.log_pa.debug("Left token: " + tok[0].data)
+            self.log_pa.debug("Right token: "+ tok[2].data)
+            n = PrefixNode(".")
+            n.left = tok[0]
+            n.right = tok[2]
             return n
         else:
-            self.log_pa.debug("Token: "+tok[0].data)
+            self.log_pa.debug("Token: " + tok[0].data)
             return tok[0]
 
     def _create_choice(self,string,loc,tok):
-        self.log_pa.debug("Start")
-        self.log_pa.debug("Tokens: "+str( len(tok) ))
+        self.log_pa.debug("Tokens: " + str(len(tok)))
         if not tok[0] is None:
             if len(tok) <3:
-                self.log_pa.debug("Token: "+tok[0].data)
+                self.log_pa.debug("Token: "+ tok[0].data)
                 return tok[0]
             else:
-                self.log_pa.debug("Left token: "+tok[0].data)
-                self.log_pa.debug("Right token: "+tok[2].data)
-
-                n = ChoiceNode("+", "choice")
+                self.log_pa.debug("Left token: " + tok[0].data)
+                self.log_pa.debug("Right token: " + tok[2].data)
+                n = ChoiceNode("+")
                 n.left = tok[0]
                 n.right = tok[2]
                 return n
 
-
     def _create_coop(self,string, loc, tok):
         if not tok[0] is None:
             if len(tok) <3:
@@ -95,12 +93,12 @@ class PEPAParser(object):
                 self.log_pa.debug("Number of tokens" + str(len(tok)))
                 self.log_pa.debug("Left token: "+tok[0].data)
                 if tok[1].actionset is not None:
-                    n = CoopNode("<"+str(tok[1].actionset)+">", "coop")
+                    n = CoopNode("<"+str(tok[1].actionset)+">")
                     n.cooptype = "sync"
                     n.actionset = tok[1].actionset
                     self.log_pa.debug(n.actionset)
                 else:
-                    n = CoopNode("||", "coop")
+                    n = CoopNode("||")
                     n.cooptype = "par"
                 if type(tok[2]).__name__ == "str":
                     self.log_pa.debug("String: "+tok[2])
@@ -113,26 +111,29 @@ class PEPAParser(object):
     def _create_sync_set(self,string, loc, tok):
         if tok[0] != "||" and tok[0] != "<>":
             self.log_pa.debug("Non empty synset: " + str(tok))
-            n = SyncsetNode("<>", "syncset")
-            n.actionset = tok
+            n = SyncsetNode("<>")
+            if "tau" in tok[0]:
+                 message = "tau present in the cooperation set"
+                 raise InvalidActionTypeError(message)
+            n.actionset = tok[0]
         else:
             self.log_pa.debug("Parallel")
-            n = SyncsetNode("||", "syncset")
+            n = SyncsetNode("||")
             n.actionset = None
         return n
 
     def _create_subtree_aggregation(self, num, procname):
         """ Transforms Process[num] into AST subtree """
-        first = CoopNode("||", "coop")
+        first = CoopNode("||")
         first.cooptype = "par"
         last = first
         for i in range(2, num+1):
-            nl = ProcdefNode(procname, "procdef")
+            nl = ProcdefNode(procname)
             if i == num:
-                nr = ProcdefNode(procname, "procdef")
+                nr = ProcdefNode(procname)
                 last.right = nr
             else:
-                nr = CoopNode("||", "coop")
+                nr = CoopNode("||")
                 nr.cooptype = "par"
                 last.right = nr
             last.left = nl
@@ -146,9 +147,8 @@ class PEPAParser(object):
             self.log_pa.debug("Non terminal - passing")
             return tok[0]
         else:
-            if tok[0].asttype == "procdef":
-                n = ProcdefNode(tok[0].data, tok[0].asttype)
-                # TODO create subtree
+            if tok[0].asttype == ProcdefNode.asttype:
+                n = ProcdefNode(tok[0].data)
                 if tok[0].aggregation == True and n.aggr_num > 1:
                     n.aggregation = True
                     n.aggr_num = tok[0].aggr_num
@@ -157,8 +157,8 @@ class PEPAParser(object):
                     n.data = "||"
                     n.cooptype = "par"
                     n.actionset = None
-            elif tok[0].asttype == "activity":
-                n = ActivityNode(tok[0].data, tok[0].asttype)
+            elif tok[0].asttype == ActivityNode.asttype:
+                n = ActivityNode(tok[0].data)
                 n.rate = tok[0].rate
                 n.action = tok[0].action
             else:
@@ -167,26 +167,27 @@ class PEPAParser(object):
             self.log_pa.debug("Token: "+tok[0].data)
         return n
 
-
     def _create_system_equation(self, string, loc, tok):
         self.log_pa.debug("Creating system EQ")
         self._systemeq = tok[0]
 
-
     def _assign_var(self,toks):
-        self._var_stack[toks[0]] = toks[2]
+        result = self.rate_parser.parse_rate_expr("".join(toks))
+        if toks[0] not in self._var_stack:
+            self._var_stack[toks[0]] = str(result)
+        else:
+            raise VariableAlreadyDefinedError("Variable {} has been already defined".format(toks[0]))
 
     def _check_var(self,str,loc,tok):
         try:
-            # just a number
             float(tok[0])
         except:
-            try:
-                if tok[0] not in ("infty", "T", "tau"):
-                    self._var_stack[tok[0]]
-            except:
-                self.log_pa.error("Rate " + tok[0]+ " not defined")
-                exit(1)
+            pass
+        else:
+            return
+        if tok[0] not in ("infty", "T"):
+            if tok[0] not in self._var_stack:
+                raise VariableNotDefinedError("Rate {} not defined in {}".format(tok[0], loc))
 
     def gramma(self):
 ## Tokens
@@ -194,7 +195,8 @@ class PEPAParser(object):
         prefix_op = Literal('.')
         choice_op = Literal('+')
         parallel = Literal("||") | Literal("<>")
-        ratename = Word(alphas.lower(),alphanums+"_")
+        tau = Literal('tau')
+        actiontype = tau | Word(alphas.lower(),alphanums+"_")
         lpar = Literal('(').suppress()
         rpar = Literal(')').suppress()
         lsqpar = Literal('[').suppress()
@@ -203,21 +205,18 @@ class PEPAParser(object):
         define = Literal('=')
         semicol = Literal(';').suppress()
         col = Literal(',').suppress()
-        number = Word(nums)
-        integer = number
-        floatnumber = Combine( integer + Optional( point + Optional(number)))
+        integer = Word(nums)
+        floatnumber = Combine( integer + Optional( point + Optional(integer)))
         passiverate = Word('infty') | Word('T')
-        internalrate = Word('tau')
         pound = Literal('#').suppress()
         percent = Literal('%').suppress()
-        peparate = (ratename | floatnumber | internalrate | passiverate).setParseAction(self._check_var)
-        peparate_indef = floatnumber | internalrate | passiverate
-        sync = Word('<').suppress() + ratename + ZeroOrMore(col + ratename) + Word('>').suppress()
+        peparate = (actiontype | floatnumber | passiverate).setParseAction(self._check_var)
+        sync = Word('<').suppress() + actiontype+ ZeroOrMore(col +actiontype) + Word('>').suppress()
         coop_op = (parallel | sync).setParseAction(self._create_sync_set)
-        activity = (ratename + col + peparate).setParseAction(self._create_activity)
+        activity = (actiontype + col + peparate).setParseAction(self._create_activity)
         procdef = (Word(alphas.upper(), alphanums+"_"+"`"+"'") + Optional(lsqpar + peparate + rsqpar)).setParseAction(self._create_procdef)
 ## RATES Definitions
-        ratedef = (Optional(percent)+ratename + define + peparate_indef).setParseAction(self._assign_var) + semicol
+        ratedef = Optional(percent)+(self.rate_parser.grammar()).setParseAction(self._assign_var) + semicol
 
         prefix = Forward()
         choice = Forward()
@@ -238,7 +237,16 @@ class PEPAParser(object):
         return pepa
 
     def parse(self,string):
-        self.gramma().parseString(string)
+        try:
+            self.gramma().parseString(string, parseAll=True)
+        except ParseException as e:
+            raise
+        for seen, location_info in self._seen.items():
+            if seen not in self._processes:
+                line = lineno(*location_info)
+                column = col(*location_info)
+                message = "{} process not defined - possible deadlock, line {}, col {}".format(seen,line, column)
+                raise ProcessNotDefinedError(message)
         return (self._processes, self._var_stack, self._systemeq, self._actions)
 
 
